package com.qmth.themis.task.quartz;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBOrg;
import com.qmth.themis.business.enums.ExamModeEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.RedisUtil;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description: 考试统计
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2023/10/27
 */
@Component
public class ExamSummaryJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(ExamSummaryJob.class);

    @Resource
    ThemisCacheService themisCacheService;

    @Resource
    TEExamSummaryService teExamSummaryService;

    @Resource
    TEExamService teExamService;

    @Resource
    TEExamActivityService teExamActivityService;

    @Resource
    TEOrgSummaryService teOrgSummaryService;

    @Resource
    TERegionSummaryService teRegionSummaryService;

    @Resource
    RedisUtil redisUtil;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        log.info("ExamSummaryJob进来了,context:{}", context);
        Map<String, List<String>> examListMap = redisUtil.getHashEntries(SystemConstant.TODAY_EXAM_LIST_MAP_CACHE);
        //获取当天考试列表
        if (!CollectionUtils.isEmpty(examListMap)) {
            LinkedMultiValueMap<Long, Long> orgExamIdMap = new LinkedMultiValueMap<>();
            for (Map.Entry<String, List<String>> entry : examListMap.entrySet()) {
                Long examId = Long.parseLong(entry.getKey());
                ExamCacheBean examCacheBean = teExamService.getExamCacheBean(examId);
                //不在考试时间范围内或者禁用，则删除当前考试批次id，更新缓存
                if (examCacheBean.getEnable().intValue() == 0 || examCacheBean.getEndTime().longValue() <= System.currentTimeMillis()) {
                    themisCacheService.removeTodayExamListCache(entry.getKey());
                    continue;
                }
                TBOrg tbOrg = themisCacheService.addOrgCache(examCacheBean.getOrgId());
                orgExamIdMap.add(tbOrg.getId(), examId);

                Map<String, List<String>> map = themisCacheService.getTodayExamListCache(entry.getKey());
                if (!CollectionUtils.isEmpty(map)) {
                    map.forEach((k, v) -> {
                        if (!CollectionUtils.isEmpty(v)) {
                            for (String s : v) {
                                JSONObject jsonObject = JSONObject.parseObject(s);
                                Long examActivityId = jsonObject.getLong(SystemConstant.EXAM_ACTIVITY_ID);
                                String roomCodeStr = jsonObject.getString(SystemConstant.ROOM_CODE_SET);
                                roomCodeStr = roomCodeStr.replaceAll("\\[", "").replaceAll("\\]", "");
                                String[] strs = roomCodeStr.split(", ");
                                Set<String> roomCodeSet = new HashSet<>(Arrays.asList(strs));
                                //换算开始时间和结束时间
                                Long startTime = null, endTime = null;
                                ExamActivityCacheBean ac = teExamActivityService.getExamActivityCacheBean(examActivityId);
                                if (ExamModeEnum.ANYTIME.equals(examCacheBean.getMode())) {
                                    startTime = ac.getStartTime() - (ac.getPrepareSeconds() * 1000);
                                    endTime = ac.getFinishTime();
                                    endTime = endTime + SystemConstant.FINISH_DELAY_TIME;//额外加5分钟，有可能后台交卷未完成
                                } else {
                                    startTime = ac.getStartTime() - (ac.getPrepareSeconds() * 1000);
                                    Integer openingSecondsTemp = ac.getOpeningSeconds();
                                    openingSecondsTemp = Objects.nonNull(openingSecondsTemp) && openingSecondsTemp.intValue() == 0 ? SystemConstant.DEFAULT_OPENING_SECONDS : openingSecondsTemp;
                                    endTime = ac.getStartTime() + (openingSecondsTemp * 1000);
                                    endTime = endTime + SystemConstant.FINISH_DELAY_TIME;//额外加5分钟，有可能后台交卷未完成
                                }
                                long timestamp = System.currentTimeMillis();
                                //当考试场次开始时间和场次结束时间在范围内，则统计，否则删除考试批次缓存列表，考试批次统计缓存信息
                                if (ac.getEnable().intValue() == 1 && startTime <= timestamp && endTime >= timestamp) {
                                    teExamSummaryService.examSummary(examId, examActivityId, roomCodeSet);
                                    teRegionSummaryService.regionSummary();
                                } else {
                                    themisCacheService.removeTodayExamListCache(entry.getKey(), examActivityId);
                                }
                            }
                        } else {
                            themisCacheService.removeTodayExamListCache(entry.getKey());
                        }
                    });
                }
            }
            //统计机构信息
            orgExamIdMap.forEach((k, v) -> {
                teOrgSummaryService.orgSummary(k, v);
            });
            teOrgSummaryService.orgSummary(0L, null);
        }
    }
}
